#!/usr/bin/env python
# -*- coding: utf-8 -*-
########################################################################
#
# importer.py
# $Id: importer.py,v 1.2 2010/08/11 00:30:22 elias Exp $
#
# Importieren der Daten im Unterverzeichnis xml/
# Dieser Code erhält keine Schönheitspreise, aber das geparste XML
# auch nicht gerade... ;-)
#
# Lizenziert unter Piratenlizenz, Share and Enjoy, but don't sue me!
# http://wwwut.wordpress.com/lizenz/
#
########################################################################
#
# Konfiguration (anpassen!)
#
########################################################################
#
DBNAME='tagesschau'
DBUSER='tagesschau'
DBPASS='tagesschau'
DBHOST='localhost'
XMLDIR='./xml'
#
########################################################################
#
# Und los gehts! Hier muss nichts mehr angepasst werden...
# (Der Vorgang kann übrigens ein *bisschen* dauern...)
#
########################################################################

import os
import os.path
import xml.dom.minidom
import warnings
with warnings.catch_warnings():
    warnings.simplefilter('ignore')
    import MySQLdb

DB = MySQLdb.connect(db=DBNAME, user=DBUSER, 
                     host=DBHOST, passwd=DBPASS,
                     charset='utf8')

class Meldung:
    def __init__(self, path):
        self.shorttitle = u''
        self.title = u''
        self.teaser = u''
        self.paragraphs = []
        self.pubdate = u''
        self.keywords = []
        self.id = u''
        d = xml.dom.minidom.parse(path)
        prop = d.getElementsByTagName('properties')
        if len(prop) > 0:
            self.__handleProperties(prop[0])
        childs = d.getElementsByTagName('childNodes')
        if len(childs) > 0:
            self.__handleChildNodes(childs[0])
        fields = d.getElementsByTagName('fields')
        if len(fields) > 0:
            self.__handleFields(fields[0])
        d.unlink()

    def __readValue(self, propitem):
        rv = u''
        v = propitem.getElementsByTagName('value')[0]
        if len(v.childNodes) > 0:
            tn = v.childNodes[0]
            if tn.nodeType == tn.TEXT_NODE:
                rv = tn.data
        return rv

    def __handleProperties(self, prop):
        for p in prop.getElementsByTagName('property'):
            n = p.getAttribute('name')
            if n == u'sophora-content:shortheadline':
                assert self.title == u''
                self.title = self.__readValue(p)
            elif n == u'sophora:creationDate':
                assert self.pubdate == u''
                self.pubdate = self.__readValue(p)
            elif n == u'sophora-content:shorttext':
                assert self.teaser == u''
                self.teaser = self.__readValue(p)
            elif n == u'sophora-content:shorttitle':
                assert self.shorttitle == u''
                self.shorttitle = self.__readValue(p)
            elif n == u'sophora:id':
                assert self.id == u''
                self.id = self.__readValue(p)

    def __handleChildNodes(self, childs):
        for cn in childs.getElementsByTagName('childNode'):
            n = cn.getAttribute('nodeType')
            if n.endswith(':copytext'):
                self.__handleCopytext(cn)

    def __handleCopytext(self, ct):
        ctcns = ct.getElementsByTagName('childNodes')[0]
        for cn in ctcns.getElementsByTagName('childNode'):
            n = cn.getAttribute('nodeType')
            if n.endswith(':paragraph'):
                self.__handleParagraphNode(cn)

    def __handleParagraphNode(self, cn):
        pns = cn.getElementsByTagName('properties')[0]
        for pn in pns.getElementsByTagName('property'):
            n = pn.getAttribute('name')
            if n.endswith(':text'):
                self.paragraphs.append(self.__readValue(pn))

    def __handleFields(self, fields):
        for t in fields.getElementsByTagName('tags'):
            tn = t.childNodes[0]
            if tn.nodeType == tn.TEXT_NODE:
                tstr = tn.data 
                self.keywords.extend(tstr.split())

def create_tables():
    print "Erzeugen der Datenbanktabellen..."
    c = DB.cursor()
    c.execute(u"""\
    DROP TABLE IF EXISTS kategorie
    """)
    c.execute(u"""\
    CREATE TABLE IF NOT EXISTS kategorie (
    kat_id int(10) unsigned NOT NULL auto_increment COMMENT 'PK',
    kat_name varchar(64) NOT NULL COMMENT 'Bezeichnung der Kategorie',
    kat_count int(10) unsigned default NULL COMMENT 'Anzahl Meldungen für die Kategorie',
    PRIMARY KEY  (kat_id),
    UNIQUE KEY kat_name (kat_name),
    KEY kat_count (kat_count)
    ) ENGINE=MyISAM DEFAULT CHARSET=utf8 COMMENT='Meldungskategorien' AUTO_INCREMENT=1
    """)
    c.execute(u"""\
    DROP TABLE IF EXISTS meldung
    """)
    c.execute(u"""\
    CREATE TABLE IF NOT EXISTS `meldung` (
    meld_id int(10) unsigned NOT NULL auto_increment COMMENT 'PK',
    meld_pubdatum datetime NOT NULL COMMENT 'Zeitpunkt der Meldung',
    meld_kurztitel varchar(255) default NULL COMMENT 'Der Kurztitel der Meldung',
    meld_internal_id varchar(16) NOT NULL COMMENT 'Interne ID aus dem CMS',
    meld_kurztext text NOT NULL COMMENT 'Teaser der Meldung (mit HTML-Markup)',
    meld_text text NOT NULL COMMENT 'Text der Meldung (ohne HTML-Markup)',
    meld_titel varchar(255) NOT NULL COMMENT 'Titel der Meldung',
    PRIMARY KEY  (meld_id),
    KEY meld_pubdatum (meld_pubdatum),
    FULLTEXT KEY meld_kurztext (meld_kurztext, meld_text, meld_titel)
) ENGINE=MyISAM DEFAULT CHARSET=utf8 COMMENT='Die Meldungen' AUTO_INCREMENT=1
    """)
    c.execute(u"""\
    DROP TABLE IF EXISTS v_kat_meld
    """)
    c.execute(u"""\
    CREATE TABLE IF NOT EXISTS v_kat_meld (
    v_kat int(10) unsigned NOT NULL COMMENT 'PK der Meldung',
    v_meld int(10) unsigned NOT NULL COMMENT 'PK der Kategorie',
    PRIMARY KEY  (v_kat, v_meld),
    KEY `v_meld` (v_meld)
) ENGINE=MyISAM DEFAULT CHARSET=utf8 COMMENT='Zuordnung der Kategorien zu den Meldungen'
    """)

theTagList = {}
theTagCursor = DB.cursor()
def associate_tag(meld_id, tag_name):
    global theTagList, theTagCursor
    tag_name = tag_name.lower()
    if theTagList.has_key(tag_name):
        tag_id = theTagList[tag_name]
    else:
        theTagCursor.execute(u"""\
          INSERT 
          INTO kategorie (kat_name)
          VALUES (%s)
        """, (tag_name, ))
        tag_id = DB.insert_id()
        theTagList[tag_name] = tag_id
    theTagCursor.execute(u"""\
      INSERT 
      INTO v_kat_meld (v_kat, v_meld)
      VALUES (%s, %s)
    """, (tag_id, meld_id))

theMeldCursor = DB.cursor()
def handle_file(path):
    global theMeldCursor
    print "Import von", path,
    try:
        im = Meldung(path)
        if len(im.paragraphs) == 0 and len(im.teaser) == 0:
            print "Kein Text"
            return
        parags = u'\n\n'.join(im.paragraphs)
        theMeldCursor.execute(u"""
          INSERT
          INTO meldung (meld_pubdatum, meld_kurztitel, meld_internal_id,
                        meld_kurztext, meld_text, meld_titel)
          VALUES       (%s, %s, %s, %s, %s, %s)
        """, (im.pubdate, im.shorttitle, im.id, im.teaser, parags, im.title))
        meld_id = DB.insert_id()
        already = {}
        for t in im.keywords:
            if already.has_key(t):
                continue
            if t.isdigit():
                continue
            associate_tag(meld_id, t)
            already[t] = True
        print "Okay"
        DB.commit()
    except:
        # Totschlag-Except für alle Fälle, nicht zur Nachahmung
        # empfohlen, aber hey, wo ist kein Dreck drin?!
        print "Failed"
        DB.rollback() # Ja, *das* geht mit einer MySQLdb nicht...

def handle_directory(directory):
    for filename in os.listdir(directory):
        filepath = os.path.join(directory, filename)
        if os.path.isdir(filepath):
            handle_directory(filepath)
        else:
            (forget_me, extension) = os.path.splitext(filepath)
            if extension.lower() == '.xml':
                handle_file(filepath)

def tag_count():
    print "Zählungen für die Tags..."
    c1 = DB.cursor()
    c2 = DB.cursor()
    c1.execute(u"""\
      SELECT v_kat, COUNT(*)
      FROM v_kat_meld
      GROUP BY 1
    """);
    for (kat_id, count) in c1.fetchall():
        c2.execute(u"""
          UPDATE kategorie
          SET kat_count = %s
          WHERE kat_id = %s
        """, (count, kat_id))
    
if __name__ == '__main__':
    create_tables()
    handle_directory(XMLDIR)
    tag_count()

########################################################################
#
# Von der Allgemeinheit über die Schundfunkgebühr bezahlte Inhalte aus
# dem frei zugänglichen Internet zu löschen, damit die Zeitungen und
# ihre Verleger ein Alleinstellungsmerkmal in deutschen Internet er-
# halten, das ist keinen Deut anders als die mittelalterliche Ver-
# brennung so genannter "Ketzer" und ihrer Schriften, um der Institution 
# Kirche ein Alleinstellungsmerkmal auf dem religiösen Markt zu er-
# halten. Es ist vielleicht weniger mörderisch, aber nicht besser oder
# weniger schamlos. Ich hoffe, dass man überall bemerken wird, was von
# einer so vorgehenden Journaille zu halten ist und diese Journaille
# und alle ihre Produkte so weit wie nur möglich meidet.
#
# Elias Schwerdtfeger, http://www.tamagothi.de/
#
########################################################################
